#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace HeuristicLab.Core {
  /// <summary>
  /// Class for storing meta-information about parameters of an operator.
  /// </summary>
  public class Parameter : ItemBase, IParameter {
    private string myName;
    /// <summary>
    /// Gets or sets the name of the current instance.
    /// </summary>
    /// <remarks>Calls <see cref="OnActualNameChanged"/> in the setter.</remarks>
    public string Name {
      get { return myName; }
    }
    private string myDescription;
    /// <summary>
    /// Gets the description of the current instance.
    /// </summary>
    public string Description {
      get { return myDescription; }
    }
    private Type myDataType;
    /// <summary>
    /// Gets the data type of the parameter.
    /// </summary>
    public Type DataType {
      get { return myDataType; }
    }
    private ParameterType myType;
    /// <summary>
    /// Gets the type of the parameter (input, output, both).
    /// </summary>
    public ParameterType Type {
      get { return myType; }
    }

    /// <summary>
    /// Initializes a new instance of <see cref="Parameter"/> with name "Anonymous",
    /// no description, and a <c>null</c> data type. The type of 
    /// the parameter is input.
    /// </summary>
    public Parameter() {
      myName = "Anonymous";
      myDescription = "";
      myDataType = null;
      myType = ParameterType.In;
    }
    /// <summary>
    /// Initializes a new instance of <see cref="Parameter"/> with the given parameters.
    /// </summary>
    /// <remarks>Calls <see cref="Parameter()"/>.</remarks>
    /// <param name="name">The name of the current instance.</param>
    /// <param name="description">The description of the current instance.</param>
    /// <param name="dataType">The data type of the parameter.</param>
    /// <param name="type">The type of the parameter.</param>
    public Parameter(string name, string description, Type dataType, ParameterType type)
      : this() {
      myName = name;
      myDescription = description;
      myDataType = dataType;
      myType = type;
    }

    /// <summary>
    /// Creates a new instance of <see cref="ParameterView"/> to represent the current instance
    /// visually.
    /// </summary>
    /// <returns>The created view as <see cref="ParameterView"/>.</returns>
    public override IView CreateView() {
      return new ParameterView(this);
    }

    /// <summary>
    /// Clones the current instance (deep clone).
    /// </summary>
    /// <param name="clonedObjects">Dictionary of all already cloned objects. (Needed to avoid cycles.)</param>
    /// <returns>The cloned object as <see cref="Parameter"/>.</returns>
    public override object Clone(IDictionary<Guid, object> clonedObjects) {
      Parameter clone = new Parameter();
      clonedObjects.Add(Guid, clone);
      clone.myName = Name;
      clone.myDescription = Description;
      clone.myDataType = DataType;
      clone.myType = Type;
      return clone;
    }

    #region Persistence Methods
    /// <summary>
    /// Saves the current instance as <see cref="XmlNode"/> in the specified <paramref name="document"/>.
    /// </summary>
    /// <remarks>Calls <see cref="StorableBase.GetXmlNode"/> of base class <see cref="ItemBase"/>. <br/>
    /// A quick overview how the single elements of the current instance are saved:
    /// <list type="bullet">
    /// <item>
    /// <term>Name: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>Name</c>.</description>
    /// </item>
    /// <item>
    /// <term>Description: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>Description</c>.</description>
    /// </item>
    /// <item><term>Data type: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>DataType</c>.</description>
    /// </item>
    /// <item>
    /// <term>Type: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>ParameterType</c>.</description>
    /// </item>
    /// </list></remarks>
    /// <param name="name">The (tag)name of the <see cref="XmlNode"/>.</param>
    /// <param name="document">The <see cref="XmlDocument"/> where to save the data.</param>
    /// <param name="persistedObjects">The dictionary of all already persisted objects. (Needed to avoid cycles.)</param>
    /// <returns>The saved <see cref="XmlNode"/>.</returns>
    public override XmlNode GetXmlNode(string name, XmlDocument document, IDictionary<Guid,IStorable> persistedObjects) {
      XmlNode node = base.GetXmlNode(name, document, persistedObjects);
      XmlAttribute nameAttribute = document.CreateAttribute("Name");
      nameAttribute.Value = Name;
      node.Attributes.Append(nameAttribute);

      XmlAttribute descriptionAttribute = document.CreateAttribute("Description");
      descriptionAttribute.Value = Description;
      node.Attributes.Append(descriptionAttribute);

      XmlAttribute dataTypeAttribute = document.CreateAttribute("DataType");
      dataTypeAttribute.Value = PersistenceManager.BuildTypeString(DataType);
      node.Attributes.Append(dataTypeAttribute);

      XmlAttribute typeAttribute = document.CreateAttribute("ParameterType");
      typeAttribute.Value = Type.ToString();
      node.Attributes.Append(typeAttribute);

      return node;
    }
    /// <summary>
    /// Loads the persisted parameter from the specified <paramref name="node"/>.
    /// </summary>
    /// <remarks>See <see cref="GetXmlNode"/> for further information on how the parameter must be 
    /// saved. <br/>
    /// Calls <see cref="StorableBase.Populate"/> of base class <see cref="ItemBase"/>.</remarks>
    /// <param name="node">The <see cref="XmlNode"/> where the parameter is saved.</param>
    /// <param name="restoredObjects">The dictionary of all already restored objects. 
    /// (Needed to avoid cycles.)</param>
    public override void Populate(XmlNode node, IDictionary<Guid,IStorable> restoredObjects) {
      base.Populate(node, restoredObjects);
      myName = node.Attributes["ActualName"].Value;
      myDescription = node.Attributes["Description"].Value;
      myDataType = System.Type.GetType(node.Attributes["DataType"].Value, true);
      myType = (ParameterType)Enum.Parse(typeof(ParameterType), node.Attributes["ParameterType"].Value);
    }
    #endregion
  }
}
